VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "Form1"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3195
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton Command1 
      Caption         =   "Click to run"
      Height          =   1335
      Left            =   1320
      TabIndex        =   0
      Top             =   840
      Width           =   2295
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
   Option Explicit


Sub Command1_Click()
'--------------------------------INDGROUND------------------------------------
'----------------A PROGRAM TO CARRY OUT INDUCTIVE GROUNDING-------------------


'This program is intended to facilitate the computation of 'inductively
'  grounded' constraints, following the method laid out in a paper by
'  B. Hayes (forthcoming) 'Phonetically-driven phonology:  the role of
'  Optimality Theory and Inductive Grounding'.

'The program reads a file (indgroun.in) that contains

'      (a) a set of phonological constraints;
'      (b) a phonetic map;
'      (c) for each constraint:
'             a listing of the locations in the phonetic map that the
'                constraint excludes
'             a listing of the neighbors of the constraint in logical
'                constraint space
'             a mnemonic label (optional)

'On this Website is the indgroun.in file that was used for the simulation
'  in the paper.

'The program outputs (to the file 'indgroun.out') a list of the constraints
'   that are grounded, with explanations attached.

'For new uses of this program, it is recommended that the user overwrite
'  the sample file with new values, since the formatting of the file has
'  to be adhered to pretty exactly.  Use a text-mode format on your word
'  processor.


'-----------------------------------------------------------------------------
DeclareAllVariables:
   
  
   'Variables needed in the first subroutine, immediately below.

   Dim CheckThisLine As String
   Dim NumberOfRowsInMap As Integer
   Dim NumberOfColumnsInMap As Integer
   Dim NumberOfRowsFoundFlag As Integer
   Dim NumberOfColumnsFoundFlag As Integer

   GoSub GetDimensionsOfPhoneticMap  'Before you can dimension the arrays, you
                                     'need the size of the phonetic map.

   Const MaxConstraints% = 100      'The maximum number of constraints the
                                    'program can deal with.  Can be increased
                                    'a bit.

   'Variables for inputting the constraint file:

      Dim DummyLine As String
      Dim DummyString As String
      Dim ConstraintName(MaxConstraints) As String
      Dim CasualLabel(MaxConstraints) As String
      Dim NeighborLabel As String
      Dim PhoneticMapLine() As String
      ReDim PhoneticMapLine(NumberOfRowsInMap + 1)

   'Data manipulated:

      Dim NumberOfConstraints As Integer
      Dim Violations() As Integer
      ReDim Violations(MaxConstraints, NumberOfRowsInMap, NumberOfColumnsInMap)
      Dim Difficulty()
      ReDim Difficulty(NumberOfRowsInMap, NumberOfColumnsInMap)
      Dim CorrectPredictions(MaxConstraints) As Integer
      Dim WrongPredictions(MaxConstraints) As Integer
      Dim Effectiveness(MaxConstraints)
      'This defines (1 vs. 0) whether one constraint is a neighbor of another:
         Dim Neighbors(MaxConstraints, MaxConstraints) As Integer

   'Labels for charts:

      Dim PhoneticMapColumnHeader() As String
      ReDim PhoneticMapColumnHeader(NumberOfColumnsInMap)
      Dim PhoneticMapRowHeader() As String
      ReDim PhoneticMapRowHeader(NumberOfRowsInMap)


   'Counters:

      Dim ConstraintIndex As Integer
      Dim RowIndex As Integer
      Dim ColumnIndex As Integer
      Dim FirstMemberRowIndex As Integer
      Dim SecondMemberRowIndex As Integer
      Dim FirstMemberColumnIndex As Integer
      Dim SecondMemberColumnIndex As Integer
      Dim NeighborIndex As Integer
      Dim OuterSortingIndex As Integer
      Dim InnerSortingIndex As Integer
      Dim BlanksIndex As Integer
      Dim PhoneticMapLineIndex As Integer
      Dim PositionIndex As Integer

   'Flags:

      Dim CasualLabelReceptivenessFlag As Integer
      Dim LoserFlag As Integer
      Dim EntryFoundFlag As Integer
      Dim CommaFlag As Integer
      Dim Winners(MaxConstraints) As Integer



'----------------------------------------------------------------------------
MainProgram:


   GoSub ReadTheInputFile
   GoSub CalculateConstraintEffectiveness
   GoSub CompareConstraintsWithNeighbors
   GoSub PrintThePhoneticMap
   GoSub PrintWordProcessorFormattedMaterial

   End


Stop
'-----------------------------------------------------------------------------
GetDimensionsOfPhoneticMap:

    'You can't interpret the file until you know the number of rows and
    '  columns in the phonetic map.

    Open "indgroun.in" For Input As #1

    Do

       Line Input #1, CheckThisLine
       If EOF(1) Then Exit Do

       If LCase$(Left$(CheckThisLine, 22)) = "number of rows in map:" Then
          Let NumberOfRowsInMap = Val(LTrim$(RTrim$(Mid$(CheckThisLine, 23))))
          Let NumberOfRowsFoundFlag = 1
       End If

       If LCase$(Left$(CheckThisLine, 25)) = "number of columns in map:" Then
          Let NumberOfColumnsInMap = Val(LTrim$(RTrim$(Mid$(CheckThisLine, 26))))
          Let NumberOfColumnsFoundFlag = 1
       End If
       
       If NumberOfRowsFoundFlag = 1 And NumberOfColumnsFoundFlag = 1 Then
          Exit Do
       End If

    Loop

    'Close the input file, because it must be opened again later:

       Close #1

    If NumberOfRowsFoundFlag = 0 Then
       Print "I cannot proceed because the input file contains no line"
       Print "  of the form 'Number of rows in map:  ***'."
       End
    End If

    If NumberOfColumnsFoundFlag = 0 Then
       Print "I cannot proceed because the input file contains no line"
       Print "  of the form 'Number of columns in map:  ***'."
       End
    End If

    Return


Stop
'------------------------------------------------------------------------------
ReadTheInputFile:

   'Open the input and output files, 'ingroun.in' and 'ingroun.out'.

      Open "indgroun.in" For Input As #1
      Open "indgroun.out" For Output As #2

    'Go through the file, and find all the constraints, along with their
    '  casual mnemonic labels.
      
      Do
         Line Input #1, CheckThisLine
         If EOF(1) Then Exit Do

         'Get the constraints, detected by how their file line begins:

         If LCase$(Left$(CheckThisLine, 11)) = "constraint:" Then

            'Keep count of how many constraints you've got:
               Let ConstraintIndex = ConstraintIndex + 1

            'Record the official name of the constraint:
               Let ConstraintName(ConstraintIndex) = LTrim$(RTrim$(Mid$(CheckThisLine, 14)))

            'Be receptive to a forthcoming casual label:
               Let CasualLabelReceptivenessFlag = 1

         'Get the casual label for the constraints:

         ElseIf LCase$(Mid$(CheckThisLine, 4, 11)) = "casual name" Then

             'Avoid cases where the user mistakenly put in two casual names:

                If CasualLabelReceptivenessFlag = 1 Then
                   Let CasualLabel(ConstraintIndex) = LTrim$(RTrim$(Mid$(CheckThisLine, 18)))
                   Let CasualLabelReceptivenessFlag = 0
                End If

         'Also, get the phonetic map in printable form:

         ElseIf LCase$(CheckThisLine) = "phonetic map:" Then

            For PhoneticMapLineIndex = 1 To NumberOfRowsInMap + 1
               Line Input #1, PhoneticMapLine(PhoneticMapLineIndex)
                  'Convert commas to blanks for intelligibility
                    For PositionIndex = 1 To Len(PhoneticMapLine(PhoneticMapLineIndex))
                       If Mid$(PhoneticMapLine(PhoneticMapLineIndex), PositionIndex, 1) = "," Then
                         Let PhoneticMapLine(PhoneticMapLineIndex) = Left$(PhoneticMapLine(PhoneticMapLineIndex), PositionIndex - 1) + " " + Mid$(PhoneticMapLine(PhoneticMapLineIndex), PositionIndex + 1)
                       End If
                    Next PositionIndex
            Next PhoneticMapLineIndex

         End If

      Loop

    'Close and reopen the file, this time to get violations and neighbors

         Close #1
         Open "indgroun.in" For Input As #1

         Let ConstraintIndex = 0

         Do
            Line Input #1, CheckThisLine
            If EOF(1) Then Exit Do
            If UCase$(Left$(CheckThisLine, 11)) = "CONSTRAINT:" Then
               Let ConstraintIndex = ConstraintIndex + 1

               Line Input #1, DummyLine
               Line Input #1, DummyLine
               Line Input #1, DummyLine

               'Read off the violations for this constraint.
               '  Violation indices:  (Constraint, Environment, FocusSegment)

                  For RowIndex = 1 To NumberOfRowsInMap
                     Input #1, DummyString
                     For ColumnIndex = 1 To NumberOfColumnsInMap
                        Input #1, DummyString
                        Select Case DummyString
                           Case "g"
                           Case "*"
                              Let Violations(ConstraintIndex, RowIndex, ColumnIndex) = 1

                           Case Else
                              Print "Problem with file at constraint #"; ConstraintIndex; ".  I need a 'g' or a '*'."
                              Print "What I got instead was "; DummyString
                              Stop
                        End Select
                     Next ColumnIndex
                        Print
                  Next RowIndex
            
            End If
         
            'Record the actual values of the phonetic map.

            If LCase$(CheckThisLine) = "phonetic map:" Then
            
               'Record the column headers:

                  For ColumnIndex = 1 To NumberOfColumnsInMap
                     Input #1, PhoneticMapColumnHeader(ColumnIndex)
                  Next ColumnIndex

               'Input the values:

                  For RowIndex = 1 To NumberOfRowsInMap

                     Input #1, PhoneticMapRowHeader(RowIndex)

                     For ColumnIndex = 1 To NumberOfColumnsInMap
                        Input #1, Difficulty(RowIndex, ColumnIndex)
                     Next ColumnIndex

                  Next RowIndex

            End If
                    
         Loop


         'You've incremented ConstraintIndex whenever you've found a constraint,
         '  so now you know how many there are.

            Let NumberOfConstraints = ConstraintIndex


   Return


Stop
'-----------------------------------------------------------------------------
CalculateConstraintEffectiveness:

    'For every constraint, find the mismatch set, form the effectiveness
    '  index, and store it in Effectiveness(MaxConstraints).

    'First, print out a header
       
       Print #2, "OUTPUT OF THE INDUCTIVE GROUNDING PROGRAM 'INDGROUN.BAS'"
       Print #2,
       Print #2, Time$; ", "; Date$
       Print #2,
       Print #2,
       Print #2, "Raw correctness scores and effectiveness values for each constraint:"
       Print #2,
       Print #2, "Constraint                  Correct Incorrect  Effectiveness =C/(I+C)"
       Print #2,
                
    For ConstraintIndex = 1 To NumberOfConstraints

       'Loop through all possible pairings, 'first member-second member',
       '  of locations in the phonetic map.

            For FirstMemberRowIndex = 1 To NumberOfRowsInMap
               For SecondMemberRowIndex = 1 To NumberOfRowsInMap
                  For FirstMemberColumnIndex = 1 To NumberOfColumnsInMap
                     For SecondMemberColumnIndex = 1 To NumberOfColumnsInMap

                     'We only want to compare non-identical entries.
                     '  For the programming ease of the doofus programmer,
                     '    we ended up making the same comparison twice in
                     '    both directions.  So you divide by two at the end.

                     'But you don't want to compare a map entry with itself,
                     '   so filter these cases out.

                     If SecondMemberRowIndex <> FirstMemberRowIndex Or SecondMemberColumnIndex <> FirstMemberColumnIndex Then

                        'Now, go through cases where a constraint might in
                        '  principle discriminate.

                        If Violations(ConstraintIndex, FirstMemberRowIndex, FirstMemberColumnIndex) = 1 Then
                           If Violations(ConstraintIndex, SecondMemberRowIndex, SecondMemberColumnIndex) = 0 Then

                              'This is a case where the first map location violates
                              '  the constraint and the second doesn't.  If the
                              '  constraint is to count as correct in this case,
                              '  the first location should be harder than the
                              '  second.

                              If Difficulty(FirstMemberRowIndex, FirstMemberColumnIndex) > Difficulty(SecondMemberRowIndex, SecondMemberColumnIndex) Then

                                 'Constraint is doing its job; increment the
                                 '  variable recording its successes.
                                 Let CorrectPredictions(ConstraintIndex) = CorrectPredictions(ConstraintIndex) + 1

                                 'But is the first location is *easier* than the second,
                                 '  record an error on the part of this constraint.

                              ElseIf Difficulty(FirstMemberRowIndex, FirstMemberColumnIndex) < Difficulty(SecondMemberRowIndex, SecondMemberColumnIndex) Then

                                  Let WrongPredictions(ConstraintIndex) = WrongPredictions(ConstraintIndex) + 1

                              End If
                           End If

                        Else   'implies that first entry obeys the constraint; it
                               '  has to do one or the other.

                           'The following code is the same, only for the opposite
                           '  case:  the first entry examined obeys the constraint,
                           '  and the second entry violates it.

                           If Violations(ConstraintIndex, SecondMemberRowIndex, SecondMemberColumnIndex) = 1 Then
                              If Difficulty(FirstMemberRowIndex, FirstMemberColumnIndex) < Difficulty(SecondMemberRowIndex, SecondMemberColumnIndex) Then
                                 Let CorrectPredictions(ConstraintIndex) = CorrectPredictions(ConstraintIndex) + 1
                              ElseIf Difficulty(FirstMemberRowIndex, FirstMemberColumnIndex) > Difficulty(SecondMemberRowIndex, SecondMemberColumnIndex) Then
                                  Let WrongPredictions(ConstraintIndex) = WrongPredictions(ConstraintIndex) + 1
                              End If
                           End If

                        End If    'This is the 'end if' for:
                                  '   'Does the first entry obey?'

                     End If       'This is the 'end if' for:  'Are the first
                                  '  and 2nd entries not the same entry?'

                     Next SecondMemberColumnIndex
                  Next FirstMemberColumnIndex
               Next SecondMemberRowIndex
            Next FirstMemberRowIndex

       'Now that you have the correctness figures, you can calculate the
       '  effectiveness index.
       
          'First, check for a zero denominator (arises when the constraint
          '  is violated by or obeyed by the whole map), and mark these
          '  special, unevaluable cases with a pseudo-effectiveness value
          '  of -1.

             If CorrectPredictions(ConstraintIndex) + WrongPredictions(ConstraintIndex) = 0 Then

                Let Effectiveness(ConstraintIndex) = -1

             Else

                'With these out of the way, you can safely compute the
                '  effectiveness values for the constraints in which it
                '  is defined.

                Let Effectiveness(ConstraintIndex) = CorrectPredictions(ConstraintIndex) / (CorrectPredictions(ConstraintIndex) + WrongPredictions(ConstraintIndex))

             End If
       
       'Print out what you have learned.

          Print #2, "*"; ConstraintName(ConstraintIndex);
          
          'Add blanks so that spacing will be even, despite variation
          '  in constraint name length.

             For BlanksIndex = Len(ConstraintName(ConstraintIndex)) To 22
                Print #2, " ";
             Next BlanksIndex
       
          Print #2, CorrectPredictions(ConstraintIndex) / 2; WrongPredictions(ConstraintIndex) / 2;
          Print #2, Effectiveness(ConstraintIndex)

    Next ConstraintIndex

    Return


Stop
'------------------------------------------------------------------------------
CompareConstraintsWithNeighbors:

    'A constraint is legal if it defeats all its neighbors.

    'Close and reopen the input file, this time to pull out the neighbors.
       
         Close #1
         Open "indgroun.in" For Input As #1

         Let ConstraintIndex = 0

         Do
            Line Input #1, CheckThisLine
            If EOF(1) Then Exit Do
            If LCase$(Left$(CheckThisLine, 11)) = "constraint:" Then

               'You've found a constraint.  Determine where you are
 
                  Let ConstraintIndex = ConstraintIndex + 1

            End If

            If LCase$(Left$(CheckThisLine, 12)) = "   neighbor:" Then
               Let NeighborLabel = LTrim$(RTrim$(Mid$(CheckThisLine, 16)))
               
               'Find out the index of this neighbor, and record it.
               '  Do a check to make sure that the neighbor has its own
               '     entry in the phonetic map.

                  Let EntryFoundFlag = 0

                  For NeighborIndex = 1 To NumberOfConstraints

                     If NeighborLabel = ConstraintName(NeighborIndex) Then

                        'Record a neighbor relation, by putting a one
                        '  in the relevant spot of the neighbor-recording
                        '  array.

                        Let Neighbors(ConstraintIndex, NeighborIndex) = 1

                        'Set a flag showing that the listed neighbor has
                        '  its own entry, as it must for the program
                        '  to work.

                           Let EntryFoundFlag = 1

                        'Now that you've found that this neighbor has its
                        '  own entry, you can quit looking

                           Exit For

                     End If      'This is the 'end if' for 'Did you find
                                 '  a constraint entry for this neighbor?'

                  Next NeighborIndex


               'Warn the user and suspend execution if there is a neighbor
               '  that has no entry in the map.

                 If EntryFoundFlag = 0 Then

                    Cls
                    Print "Warning:  no phonetic map entry exists for the neighbor"
                    Print "  constraint "; NeighborLabel; ", the neighbor of "; ConstraintName(ConstraintIndex); "."
                    Stop

                 End If
                    

            End If   'This is the 'END IF' for 'Have you found a file line
                     '  that begins with the word 'neighbor:'?'

         Loop        'End of loop:  go through the input file.
                                   

   'Now check for each constraint whether it is better than all of its neighbors

        'Print out what you are doing, as you go.  Header:

           Print #2,
           Print #2,
           Print #2, "JUSTIFICATION OF CONSTRAINTS:"

        'Loop through the constraints:

        For ConstraintIndex = 1 To NumberOfConstraints

           'First, don't bother with constraints whose effectiveness
           '  could not be computed.

              If Effectiveness(ConstraintIndex) = -1 Then

                 Print #2, "The constraint "; Chr$(2); "*"; ConstraintName(ConstraintIndex); Chr$(2); " cannot be evaluated with this map."
                 Let Winners(ConstraintIndex) = -1
                 Print #2, "   Its neighbors are:"
                 For NeighborIndex = 1 To NumberOfConstraints
                    If Neighbors(ConstraintIndex, NeighborIndex) = 1 Then
                          Print #2, "                       *"; ConstraintName(NeighborIndex)
                    End If
                 Next NeighborIndex

          'Otherwise, compare the effectiveness of the constraint against
          '  that of all of its neighbors:

              Else

                 'The LoserFlag will be hoisted when a constraint loses
                 '  to any neighbor, indicating ultimately ill-formedness
                 '  for that constraint.

                 Let LoserFlag = 0

                 'Print out a header for the constraint:

                    Print #2,
                    Print #2, "The constraint "; Chr$(2); "*"; ConstraintName(ConstraintIndex); Chr$(2); ";, value ";
                    Print #2, Effectiveness(ConstraintIndex);
                    Print #2, ", is "

                 'Compare with all neighbors:

                    For NeighborIndex = 1 To NumberOfConstraints

                       If Neighbors(ConstraintIndex, NeighborIndex) = 1 Then

                          'Agnostic outcome, interpreted as favorable:
                          If Effectiveness(NeighborIndex) = -1 Then
                             Print #2, "     not determineable w.r.t. constraint *"; ConstraintName(NeighborIndex)

                          'Positive outcome:
                          ElseIf Effectiveness(ConstraintIndex) > Effectiveness(NeighborIndex) Then
                             Print #2, "     superior to constraint *"; ConstraintName(NeighborIndex); ", value ";
                             Print #2, Effectiveness(NeighborIndex)

                          'Constraint is tied with a neighbor, interpreted as unfavorable:
                          ElseIf Effectiveness(ConstraintIndex) = Effectiveness(NeighborIndex) Then
                             Print #2, "     equal to constraint *"; ConstraintName(NeighborIndex); ", value ";
                             Print #2, Effectiveness(NeighborIndex)
                             Let LoserFlag = 1

                          'Constraint loses to neighbor:
                          ElseIf Effectiveness(ConstraintIndex) < Effectiveness(NeighborIndex) Then
                             Print #2, "     inferior to constraint *"; ConstraintName(NeighborIndex); ", value ";
                             Print #2, Effectiveness(NeighborIndex)
                             Let LoserFlag = 1

                          End If  'This is END IF for set of comparisons between
                                  '  candidate constraint and its neighbors.

                       End If     'This is END IF for:  'Is this constraint a neighbor
                                  '  of the candidate?'

                   Next NeighborIndex   'Finish going through all possible neighbors.

                'Now you know, so print out whether the constraint is a winner.

                   Print #2,
                   Print #2, "     Therefore, this constraint is ";

                   Select Case LoserFlag

                      'Constraint is a winner, with unhoisted LoserFlag:
                      Case 0
                        Print #2, Chr$(2); "legal"; Chr$(2); "."

                        'Record it in the Valhalla of winners:
                        Let Winners(ConstraintIndex) = 1

                      'Constraint is a loser:
                      Case 1
                        Print #2, Chr$(2); "illegal"; Chr$(2); "."

                   End Select
  
              End If

              'Print a blank line between entries for consecutive constraints:
              Print #2,
           
        Next ConstraintIndex



        'Since you now know the neighbors, print out a useful chart in
        '  word processing format:  constraint, effectiveness, neighbors.

        Print #2,
        Print #2,
        Print #2, "Material for word processor import:"
        Print #2,
        Print #2,
        Print #2, "List of constraints with effectiveness and neighbors:"
        Print #2,
        Print #2, "Constraint"; Chr$(9); "Effectiveness"; Chr$(9); "Neighbors"
        For ConstraintIndex = 1 To NumberOfConstraints
           Print #2, "*"; ConstraintName(ConstraintIndex);
           Print #2, Chr$(9);
           If Effectiveness(ConstraintIndex) = -1 Then
              Print #2, "(undetermined)";
           Else
              Print #2, Effectiveness(ConstraintIndex);
           End If
           Print #2, Chr$(9);

           'Now print out the neighbors.

                  Let CommaFlag = 0
                  For NeighborIndex = 1 To NumberOfConstraints

                     'Is this constraint in a neighbor relation?

                     If Neighbors(ConstraintIndex, NeighborIndex) = 1 Then

                        'CommaFlag is used to put a comma only after non-final
                        '  neighbors in the list.

                        Select Case CommaFlag
                           Case 0
                              Print #2, "*"; ConstraintName(NeighborIndex);
                              Let CommaFlag = 1
                           Case 1
                              Print #2, ", *"; ConstraintName(NeighborIndex);
                        End Select

                     End If
                  Next NeighborIndex
                  Print #2,

           Next ConstraintIndex









       'Now, sort the constraints by effectiveness, so they can be printed
       '   out in this order.

           GoSub SortTheConstraints

       'Now, you can print them.

           Print #2,
           Print #2, "The legal constraints are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = 1 Then
                 Print #2, Effectiveness(ConstraintIndex);
                 GoSub PrintMeasuredName
              End If
           Next ConstraintIndex

           Print #2,
           Print #2, "The illegal constraints are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = 0 Then
                 Print #2, Effectiveness(ConstraintIndex);
                 GoSub PrintMeasuredName
              End If
           Next ConstraintIndex

           Print #2,
           Print #2, "The constraints that cannot be evaluated are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = -1 Then
                 Print #2, "        ";
                 GoSub PrintMeasuredName
              End If
           Next ConstraintIndex
           
   Return

Stop
'-----------------------------------------------------------------------------
SortTheConstraints:

    'Sort the constraints by level of effectiveness.
    '  In this sort, we rearrange only those arrays whose values will be
    '  printed out.

    For OuterSortingIndex = 1 To NumberOfConstraints
       For InnerSortingIndex = OuterSortingIndex + 1 To NumberOfConstraints

          If Effectiveness(OuterSortingIndex) < Effectiveness(InnerSortingIndex) Then

             'SWAP ConstraintName(OuterSortingIndex), ConstraintName(InnerSortingIndex)
             'SWAP Effectiveness(OuterSortingIndex), Effectiveness(InnerSortingIndex)
             'SWAP Winners(OuterSortingIndex), Winners(InnerSortingIndex)
             'SWAP CasualLabel(OuterSortingIndex), CasualLabel(InnerSortingIndex)

          End If

       Next InnerSortingIndex
    Next OuterSortingIndex


    Return


Stop
'-----------------------------------------------------------------------------
PrintMeasuredName:

   'This subroutine is called by earlier code, and is used to produce
   '  even spacing in the output even when the constraint names are
   '  of unequal length.
   
   Print #2, "*"; ConstraintName(ConstraintIndex);
   
   For BlanksIndex = Len(ConstraintName(ConstraintIndex)) To 22
      Print #2, " ";
   Next BlanksIndex

   Print #2, CasualLabel(ConstraintIndex)

   Return


Stop
'----------------------------------------------------------------------------
PrintThePhoneticMap:

   Print #2,
   Print #2,
   Print #2, "These constraints are based on the following phonetic map:"
   Print #2,

   For PhoneticMapLineIndex = 1 To NumberOfRowsInMap + 1
      Print #2, PhoneticMapLine(PhoneticMapLineIndex)
   Next PhoneticMapLineIndex
   
   Return


Stop
'------------------------------------------------------------------------------
PrintWordProcessorFormattedMaterial:

    'Print out a version of the constraints for suitable for import into
    '   word processors.

        Print #2,
        Print #2,
        Print #2, "Material for word processor import:"
        Print #2,
        Print #2,

     'I. Phoentic map:

        Print #2, "Phonetic map:"
        Print #2,

        'Print the column headers, with a blank for the upper left corner:

           For ColumnIndex = 1 To NumberOfColumnsInMap
              Print #2, Chr$(9); PhoneticMapColumnHeader(ColumnIndex);
           Next ColumnIndex
           Print #2,

        'Print the rows, with labels and numerical entries:

           For RowIndex = 1 To NumberOfRowsInMap

              Print #2, PhoneticMapRowHeader(RowIndex);
              For ColumnIndex = 1 To NumberOfColumnsInMap
                 Print #2, Chr$(9); LTrim$(RTrim$(Str$(Difficulty(RowIndex, ColumnIndex))));
              Next ColumnIndex
              Print #2,

           Next RowIndex

           Print #2,
           Print #2,

    'II. Print the list of legal, illegal, and indeterminate constraints:

           Print #2,
           Print #2, "The legal constraints are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = 1 Then
                 Print #2, Effectiveness(ConstraintIndex);
                 Print #2, Chr$(9);
                 Print #2, ConstraintName(ConstraintIndex);
                 Print #2, Chr$(9);
                 Print #2, CasualLabel(ConstraintIndex)
              End If
           Next ConstraintIndex

           Print #2,
           Print #2, "The illegal constraints are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = 0 Then
                 Print #2, Effectiveness(ConstraintIndex);
                 Print #2, Chr$(9);
                 Print #2, ConstraintName(ConstraintIndex);
                 Print #2, Chr$(9);
                 Print #2, CasualLabel(ConstraintIndex)
              End If
           Next ConstraintIndex

           Print #2,
           Print #2, "The constraints that cannot be evaluated are:"
           Print #2,
           For ConstraintIndex = 1 To NumberOfConstraints
              If Winners(ConstraintIndex) = -1 Then
                 Print #2, "(undefined)";
                 Print #2, Chr$(9);
                 Print #2, ConstraintName(ConstraintIndex);
                 Print #2, Chr$(9);
                 Print #2, CasualLabel(ConstraintIndex)
              End If
           Next ConstraintIndex

   Return



End Sub
